@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2010,2011 The ESPResSo project
+  Copyright (C) 2010,2011,2012 The ESPResSo project
   Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 
     Max-Planck-Institute for Polymer Research, Theory Group
   
@@ -64,6 +64,9 @@
 /** granularity of the time measurement */
 #define P3M_TIME_GRAN 2
 
+/** whether the P3M charge assignment fraction is stored or not */
+#define P3M_STORE_CA_FRAC
+
 /************************************************
  * data types
  ************************************************/
@@ -94,7 +97,7 @@ typedef struct {
   int q_2_off;
   /** offset between mesh lines of the two last dimensions */
   int q_21_off;
-} local_mesh;
+} p3m_local_mesh;
 
 /** Structure for send/recv meshs. */
 typedef struct {
@@ -116,17 +119,59 @@ typedef struct {
   int r_size[6];
   /** maximal size for send/recv buffers. */
   int max;
-} send_mesh;
+} p3m_send_mesh;
+
+/** Structure to hold P3M parameters and some dependend variables. */
+typedef struct {
+  /** Ewald splitting parameter (0<alpha<1), rescaled to alpha_L = alpha * box_l. */
+  double alpha_L;
+  /** Cutoff radius for real space electrostatics (>0), rescaled to r_cut_iL = r_cut * box_l_i. */
+  double r_cut_iL;
+  /** number of mesh points per coordinate direction (>0). */
+  int    mesh[3];
+  /** offset of the first mesh point (lower left corner) from the
+      coordinate origin ([0,1[). */
+  double mesh_off[3];
+  /** charge assignment order ([0,7]). */
+  int    cao;
+  /** number of interpolation points for charge assignment function */
+  int    inter;
+  /** Accuracy of the actual parameter set. */
+  double accuracy;
+
+  /** epsilon of the "surrounding dielectric". */
+  double epsilon;
+  /** Cutoff for charge assignment. */
+  double cao_cut[3];
+  /** mesh constant. */
+  double a[3];
+  /** inverse mesh constant. */
+  double ai[3];
+  /** unscaled \ref alpha_L for use with fast inline functions only */
+  double alpha;
+  /** unscaled \ref r_cut_iL for use with fast inline functions only */
+  double r_cut;
+  /** full size of the interpolated assignment function */
+  int inter2;
+  /** number of points unto which a single charge is interpolated, i.e. p3m.cao^3 */
+  int cao3;
+  /** additional points around the charge assignment mesh, for method like dielectric ELC
+      creating virtual charges. */
+  double additional_mesh[3];
+} p3m_parameter_struct;
+
+/** initialize the parameter struct */
+void p3m_common_parameter_pre_init(p3m_parameter_struct *params);
 
 /** print local mesh content. 
     \param l local mesh structure.
 */
-void p3m_print_local_mesh(local_mesh l);
+void p3m_p3m_print_local_mesh(p3m_local_mesh l);
 
 /** print send mesh content. 
  *  \param sm send mesh structure.
 */
-void p3m_print_send_mesh(send_mesh sm);
+void p3m_p3m_print_send_mesh(p3m_send_mesh sm);
 
 /** Add values of a 3d-grid input block (size[3]) to values of 3d-grid
  *  ouput array with dimension dim[3] at start position start[3].  
@@ -137,68 +182,20 @@ void p3m_print_send_mesh(send_mesh sm);
  *  \param size        Dimensions of the block
  *  \param dim         Dimensions of the output grid.
 */
-void add_block(double *in, double *out, int start[3], int size[3], int dim[3]);
+void p3m_add_block(double *in, double *out, int start[3], int size[3], int dim[3]);
 
-/** One of the aliasing sums used by \ref P3M_k_space_error. 
+/** One of the aliasing sums used by \ref p3m_k_space_error. 
     (fortunately the one which is most important (because it converges
     most slowly, since it is not damped exponentially)) can be
     calculated analytically. The result (which depends on the order of
     the spline interpolation) can be written as an even trigonometric
     polynomial. The results are tabulated here (The employed formula
     is Eqn. 7.66 in the book of Hockney and Eastwood). */
-double analytic_cotangent_sum(int n, double mesh_i, int cao);
+double p3m_analytic_cotangent_sum(int n, double mesh_i, int cao);
 
 /** Computes the  assignment function of for the \a i'th degree
     at value \a x. */
-double P3M_caf(int i, double x,int cao_value);
-
-/* These functions are used for calculation of a charge assignment function */
-/* double caf10(double x); */
-
-/* double caf20(double x); */
-/* double caf21(double x); */
-
-/* double caf30(double x); */
-/* double caf31(double x); */
-/* double caf32(double x); */
-
-/* double caf40(double x); */
-/* double caf41(double x); */
-/* double caf42(double x); */
-/* double caf43(double x); */
-
-/* double caf50(double x); */
-/* double caf51(double x); */
-/* double caf52(double x); */
-/* double caf53(double x); */
-/* double caf54(double x); */
-
-/* double caf60(double x); */
-/* double caf61(double x); */
-/* double caf62(double x); */
-/* double caf63(double x); */
-/* double caf64(double x); */
-/* double caf65(double x); */
-
-/* double caf70(double x); */
-/* double caf71(double x); */
-/* double caf72(double x); */
-/* double caf73(double x); */
-/* double caf74(double x); */
-/* double caf75(double x); */
-/* double caf76(double x); */
-
-/* the function to calculate caf without interpolation */
-/* typedef double func(double); */
-/* func *int_caf_wi[7][7] = { {caf10}, */
-/*                             {caf20, caf21}, */
-/* 			    {caf30, caf31, caf32}, */
-/* 			    {caf40, caf41, caf42, caf43}, */
-/* 			    {caf50, caf51, caf52, caf53, caf54}, */
-/* 			    {caf60, caf61, caf62, caf63, caf64, caf65}, */
-/* 			    {caf70, caf71, caf72, caf73, caf74, caf75, caf76} */
-/*                           }; */
-
+double p3m_caf(int i, double x,int cao_value);
 
 #endif /* P3M || DP3M */
 
