@@ -1,6 +1,7 @@
 /*
-  Copyright (C) 2010,2011 The ESPResSo project
-  Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 Max-Planck-Institute for Polymer Research, Theory Group, PO Box 3148, 55021 Mainz, Germany
+  Copyright (C) 2010,2011,2012 The ESPResSo project
+  Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 
+    Max-Planck-Institute for Polymer Research, Theory Group
   
   This file is part of ESPResSo.
   
@@ -21,7 +22,6 @@
 
 #ifdef VIRTUAL_SITES_COM
 
-#include "parser.h"
 #include "virtual_sites.h"
 #include "cells.h"
 #include "topology.h"
@@ -91,7 +91,7 @@ void calc_mol_vel(Particle *p_com,double v_com[3]){
       p=local_particles[topology[mol_id].part.e[i]];
       #ifdef VIRTUAL_SITES_DEBUG
       if (p==NULL){
-         char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+         char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
          ERROR_SPRINTF(errtxt,"Particle does not exist in calc_mol_vel! id=%i\n",topology[mol_id].part.e[i]);
          return;
       }
@@ -110,7 +110,7 @@ void calc_mol_vel(Particle *p_com,double v_com[3]){
    }
 #ifdef VIRTUAL_SITES_DEBUG
    if (count!=topology[mol_id].part.n-1){
-      char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+      char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
       ERROR_SPRINTF(errtxt,"There is more than one COM in calc_mol_vel! mol_id=%i\n",mol_id);
       return;
    }
@@ -135,7 +135,7 @@ void calc_mol_pos(Particle *p_com,double r_com[3]){
       p=local_particles[topology[mol_id].part.e[i]];
       #ifdef VIRTUAL_SITES_DEBUG
       if (p==NULL){
-         char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+         char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
          ERROR_SPRINTF(errtxt,"Particle does not exist in calc_mol_pos! id=%i\n",topology[mol_id].part.e[i]);
          return;
       }
@@ -156,7 +156,7 @@ void calc_mol_pos(Particle *p_com,double r_com[3]){
    }
 #ifdef VIRTUAL_SITES_DEBUG
    if (count!=topology[mol_id].part.n-1){
-      char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+      char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
       ERROR_SPRINTF(errtxt,"There is more than one COM in calc_mol_pos! mol_id=%i\n",mol_id);
       return;
    }
@@ -216,7 +216,7 @@ void put_mol_force_on_parts(Particle *p_com){
       p=local_particles[topology[mol_id].part.e[i]];
       #ifdef VIRTUAL_SITES_DEBUG
       if (p==NULL){
-         char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+         char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
          ERROR_SPRINTF(errtxt,"Particle does not exist in put_mol_force_on_parts! id=%i\n",topology[mol_id].part.e[i]);
          return;
       }
@@ -231,7 +231,7 @@ void put_mol_force_on_parts(Particle *p_com){
       p=local_particles[topology[mol_id].part.e[i]];
       #ifdef VIRTUAL_SITES_DEBUG
       if (p==NULL){
-         char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+         char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
          ERROR_SPRINTF(errtxt,"Particle does not exist in put_mol_force_on_parts! id=%i\n",topology[mol_id].part.e[i]);
          return;
       }
@@ -247,7 +247,7 @@ void put_mol_force_on_parts(Particle *p_com){
    }
 #ifdef VIRTUAL_SITES_DEBUG
    if (count!=topology[mol_id].part.n-1){
-      char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+      char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
       ERROR_SPRINTF(errtxt,"There is more than one COM input_mol_force_on_parts! mol_id=%i\n",mol_id);
       return;
    }
@@ -260,24 +260,31 @@ Particle *get_mol_com_particle(Particle *calling_p){
    Particle *p;
 
    mol_id=calling_p->p.mol_id;
+
+   if (mol_id < 0) {
+     char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
+     ERROR_SPRINTF(errtxt,"Partice does not have a mol id! pnr=%i\n",
+		   calling_p->p.identity);
+     return NULL;
+   }
    for (i=0;i<topology[mol_id].part.n;i++){
       p=local_particles[topology[mol_id].part.e[i]];
-      #ifdef VIRTUAL_SITES_DEBUG
+
       if (p==NULL){
-         char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+         char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
          ERROR_SPRINTF(errtxt,"Particle does not exist in put_mol_force_on_parts! id=%i\n",topology[mol_id].part.e[i]);
          return NULL;
       }
-      #endif
+
       if (ifParticleIsVirtual(p)) {
           return p;
        }
    }
-#ifdef VIRTUAL_SITES_DEBUG
-   char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+
+   char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
    ERROR_SPRINTF(errtxt,"No com found in get_mol_com_particleParticle does not exist in put_mol_force_on_parts! pnr=%i\n",calling_p->p.identity);
    return NULL;
-#endif
+
    return calling_p;
 }
 
@@ -288,12 +295,12 @@ double get_mol_dist(Particle *p1,Particle *p2){
    p2_com=get_mol_com_particle(p2);
    #ifdef VIRTUAL_SITES_DEBUG
    if (p1_com==NULL){
-      char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+      char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
       ERROR_SPRINTF(errtxt,"COM Particle not found for particle in get_mol_dist id=%i\n",p1->p.identity);
       dist[0]=dist[1]=dist[2]=0.0;
    }
    if (p2_com==NULL){
-      char *errtxt = runtime_error(128 + 3*TCL_INTEGER_SPACE);
+      char *errtxt = runtime_error(128 + 3*ES_INTEGER_SPACE);
       ERROR_SPRINTF(errtxt,"COM Particle not found for particle in get_mol_dist id=%i\n",p2->p.identity);
       dist[0]=dist[1]=dist[2]=0.0;
    }
@@ -305,112 +312,14 @@ double get_mol_dist(Particle *p1,Particle *p2){
 
 /** \name Statistic Functions */
 /** \name Private Functions */
-double calc_pressure_mol(int type1,int type2);
-double calc_energy_kinetic_mol(int type);
 void calc_force_between_mol(int mol_id1,int mol_id2,double force[3]);
 #ifdef ELECTROSTATICS
-void calc_total_dipolmoment_mol(int type,double total_dipole[4]);
-void calc_absolute_dipolmoment_mol(int type,double total_dipole[2]);
 void calc_dipole_of_molecule(int mol_id,double dipole[4]);
 #endif
 
 Particle *get_mol_com_particle_from_molid_cfg(int mol_id);
 void get_mol_dist_vector_from_molid_cfg(int mol_id1,int mol_id2,double dist[3]);
 
-int tclcommand_analyze_parse_and_print_pressure_mol(Tcl_Interp *interp,int argc, char **argv)
-{
-   char buffer[TCL_DOUBLE_SPACE];
-   int type1, type2;
-   double psum;
-   #ifdef ELECTROSTATICS
-   #ifndef INTER_RF
-   Tcl_ResetResult(interp);
-   Tcl_AppendResult(interp, "parse_and_print_pressure_mol is only possible with INTER_RF ", (char *)NULL);
-   return (TCL_ERROR);
-   #endif
-   #endif
-   updatePartCfg(WITHOUT_BONDS);
-   if (!sortPartCfg()) {
-      char *errtxt = runtime_error(128);
-      ERROR_SPRINTF(errtxt, "{059 parse_and_print_pressure_mol: could not sort particle config, particle ids not consecutive?} ");
-      return TCL_ERROR;
-   }
-   if (argc < 2) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze pressure_mol <type1> <type2>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-
-   if (!ARG0_IS_I(type1)) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze pressure_mol <type1> <type2>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   if (!ARG1_IS_I(type2)) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze pressure_mol <type1> <type2>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   argc-=2; argv+=2;
-
-   if (n_molecules==0) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "No molecules defined !", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   psum=calc_pressure_mol(type1,type2);
-   //sprintf(buffer,"%i",type1);
-   //Tcl_AppendResult(interp,"{ analyze pressure_mol ",buffer," ",(char *)NULL);   
-   //sprintf(buffer,"%i",type2);
-   //Tcl_AppendResult(interp,buffer," ",(char *)NULL);   
-   sprintf(buffer,"%e",psum);
-   Tcl_AppendResult(interp, buffer,(char *)NULL);
-   return TCL_OK;
-}
-
-int tclcommand_analyze_parse_and_print_energy_kinetic_mol(Tcl_Interp *interp,int argc, char **argv)
-{
-   char buffer[TCL_DOUBLE_SPACE];
-   int type;
-   double Ekin;
-   updatePartCfg(WITHOUT_BONDS);
-   if (!sortPartCfg()) {
-      char *errtxt = runtime_error(128);
-      ERROR_SPRINTF(errtxt, "{059 parse_and_print_energy_kinetic_mol: could not sort particle config, particle ids not consecutive?} ");
-      return TCL_ERROR;
-   }
-   if (argc < 1) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze energy_kinetic <type>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-
-   if (!ARG0_IS_I(type)) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze energy_kinetic <type>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   argc-=1; argv+=1;
-
-   if (n_molecules==0) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "No molecules defined !", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   Ekin=calc_energy_kinetic_mol(type);
-   if (Ekin < 0.0) {
-     Tcl_ResetResult(interp);
-      sprintf(buffer,"%i",-(int)Ekin);
-      Tcl_AppendResult(interp,"Could not fetch com in calc_energy_kinetic_mol! From mol_id",buffer, (char *)NULL);
-      return (TCL_ERROR);
-   }
-   //sprintf(buffer,"%i",type);
-   //Tcl_AppendResult(interp,"{ analyze pressure_mol ",buffer," ",(char *)NULL);   
-   sprintf(buffer,"%e",Ekin);
-   Tcl_AppendResult(interp, buffer,(char *)NULL);
-   return TCL_OK;
-}
-
 double calc_pressure_mol(int type1,int type2){
   double force[3], com_dist[3],psum=0;
   int i,j,k,l,start;
@@ -440,70 +349,6 @@ double calc_pressure_mol(int type1,int type2){
   return psum;
 }
 
-int tclcommand_analyze_parse_and_print_dipmom_mol(Tcl_Interp *interp,int argc, char **argv)
-{
-#ifndef ELECTROSTATICS
-   Tcl_ResetResult(interp);
-   Tcl_AppendResult(interp, "calc_dipole_mol is not possible without ELECTROSTATICS", (char *)NULL);
-   return (TCL_ERROR);
-#else
-   int k,type;
-   char buffer[TCL_DOUBLE_SPACE];
-   double dipole[4];
-   updatePartCfg(WITHOUT_BONDS);
-   if (!sortPartCfg()) {
-      char *errtxt = runtime_error(128);
-      ERROR_SPRINTF(errtxt, "{059 parse_and_print_dipole: could not sort particle config, particle ids not consecutive?} ");
-      return TCL_ERROR;
-   }
-   if (n_molecules==0) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "No molecules defined !", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   if (argc < 2) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze parse_and_print_dipole_mol <type>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-
-   if (!ARG1_IS_I(type)) {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "usage: analyze parse_and_print_dipole_mol <type>", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   if (ARG0_IS_S("total")){
-      calc_total_dipolmoment_mol(type,dipole);
-      sprintf(buffer,"%i ",type);
-      Tcl_AppendResult(interp,"{ dipolemoment_mol total ",buffer,(char *)NULL);
-      for (k=0;k<3;k++)
-      {
-            sprintf(buffer,"%e ",dipole[k]);
-            Tcl_AppendResult(interp, buffer,(char *)NULL);
-      }
-      sprintf(buffer,"%e",dipole[3]);
-      Tcl_AppendResult(interp,buffer,"}",(char *)NULL);
-   }
-   else if (ARG0_IS_S("absolute")){
-      calc_absolute_dipolmoment_mol(type,dipole);
-      sprintf(buffer,"%i ",type);
-      Tcl_AppendResult(interp,"{ dipolemoment_mol absolute ",buffer,(char *)NULL);
-      sprintf(buffer,"%e ",dipole[0]);
-      Tcl_AppendResult(interp, buffer,(char *)NULL);
-      sprintf(buffer,"%e",dipole[1]);
-      Tcl_AppendResult(interp,buffer,"}",(char *)NULL);
-   }
-   else
-   {
-      Tcl_ResetResult(interp);
-      Tcl_AppendResult(interp, "Feature not implemented", (char *)NULL);
-      return (TCL_ERROR);
-   }
-   argc-=2; argv+=2;
-   return TCL_OK;
-#endif
-}
-
 void calc_force_between_mol(int mol_id1,int mol_id2,double force[3]){
   int i,j;
   Particle *p1,*p2;
@@ -670,44 +515,4 @@ void get_mol_dist_vector_from_molid_cfg(int mol_id1,int mol_id2,double dist[3]){
    get_mi_vector(dist,p1_com->r.p, p2_com->r.p);
 }
 
-int tclcommand_analyze_parse_and_print_check_mol(Tcl_Interp *interp,int argc, char **argv){
-   int j,count=0;
-   double dist;
-   char buffer[TCL_DOUBLE_SPACE];
-   Particle p;
-   updatePartCfg(WITHOUT_BONDS);
-   for(j=0; j<n_total_particles; j++){
-      if (!ifParticleIsVirtual(&partCfg[j])) continue;
-      get_particle_data(j,&p);
-      //dist=min_distance(partCfg[j].r.p,p.r.p);
-      unfold_position(p.r.p,p.l.i);
-      dist=distance(partCfg[j].r.p,p.r.p);
-      if (dist > 0.01){
-         if (count==0) Tcl_AppendResult(interp,"BEGIN Particle Missmatch: \n", (char *)NULL);
-         sprintf(buffer,"%i",j);
-         Tcl_AppendResult(interp,"Particle ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,partCfg[j].r.p[0] , buffer);
-         Tcl_AppendResult(interp," partCfg x ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,partCfg[j].r.p[1] , buffer);
-         Tcl_AppendResult(interp," y ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,partCfg[j].r.p[2] , buffer);
-         Tcl_AppendResult(interp," z ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,p.r.p[0] , buffer);
-         Tcl_AppendResult(interp," my_partCfg x ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,p.r.p[1] , buffer);
-         Tcl_AppendResult(interp," y ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp,p.r.p[2] , buffer);
-         Tcl_AppendResult(interp," z ",buffer, (char *)NULL);
-         Tcl_PrintDouble(interp, dist, buffer);
-         Tcl_AppendResult(interp," dist ",buffer,"\n", (char *)NULL);
-         count++;
-      }
-   }
-   if (count!=0){
-      Tcl_AppendResult(interp,"END Particle Missmatch\n", (char *)NULL);
-      return(TCL_ERROR);
-   }
-   return(TCL_OK);
-}
-
 #endif
