@@ -1,4 +1,4 @@
-# Copyright (C) 2010,2011 The ESPResSo project
+# Copyright (C) 2010,2011,2012 The ESPResSo project
 # Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 
 #    Max-Planck-Institute for Polymer Research, Theory Group
 #  
@@ -149,7 +149,7 @@ if { [catch {
 	set de [expr abs([lindex [lindex $energy $i] 3]-[lindex [lindex $new_energy $i] 3]) ]
 	if { $de > $maxde } { set maxde $de }
 	if { $de > $epsilon } {
-	    puts "Energy Error for [lindex $new_energy $i]: deviation $maxde"
+	    puts "Energy Error for [lindex $new_energy $i] vs. [lindex $energy $i]: deviation $maxde"
 	    #	    error "energy error too large"
 	}
     }
