@@ -1,4 +1,4 @@
-# Copyright (C) 2010,2011 The ESPResSo project
+# Copyright (C) 2010,2011,2012 The ESPResSo project
 # Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 
 #   Max-Planck-Institute for Polymer Research, Theory Group
 #  
@@ -122,9 +122,9 @@ if { [catch {
     }
     inter ljforcecap 0
 
-    # maggs requires domain decompostion with no verlet lists
+    # memd requires domain decompostion with no verlet lists
     cellsystem domain_decomposition -no_verlet_list
-    inter coulomb $bjerrum maggs $f_mass $mesh
+    inter coulomb $bjerrum memd $f_mass $mesh
 
     set act_min_dist [analyze mindist]
 
@@ -191,3 +191,5 @@ if { [catch {
 } res ] } {
     error_exit $res
 }
+
+exit 0
\ No newline at end of file
